/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.plugins.blog.ui.sitemap.Update');

org.ametys.plugins.blog.ui.sitemap.Update.initialized = false;

org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId = null;
org.ametys.plugins.blog.ui.sitemap.Update.createdPostIds = [];
org.ametys.plugins.blog.ui.sitemap.Update.deletingPostPageTargets = {};
org.ametys.plugins.blog.ui.sitemap.Update.deletingTagPageTargets = {};

org.ametys.plugins.blog.ui.sitemap.Update.initialize = function()
{
    if (org.ametys.plugins.blog.ui.sitemap.Update.initialized)
    {
        return;
    }
    
    // Initialize.
    org.ametys.messagebus.MessageBus.getInstance().register(org.ametys.plugins.blog.ui.sitemap.Update);
    
    org.ametys.plugins.blog.ui.sitemap.Update.initialized = true;
}

org.ametys.plugins.blog.ui.sitemap.Update.onMessage = function(messages)
{
    for (var i = 0; i < messages.length; i++)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
        {
            var allTargets = messages[i].getTargets();
            var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(allTargets, function(target) { return target.getType() == 'sitemap' } );
            
            if (target != null)
            {
                var id = target.getParameters().id;
                var lang = target.getParameters().lang;
                var siteName = target.getParameters().siteName;
                
                var url = '/' + siteName + '/' + lang + '/blog-root-page.xml';
                
                var serverMessage = new org.ametys.servercomm.ServerMessage('blog', url, {}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, org.ametys.plugins.blog.ui.sitemap.Update.changeSitemapCb, this, [id, name]);
                org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
            }
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_CREATED)
        {
            var allTargets = messages[i].getTargets();
            var contentTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'content' && target.getParameters().type == 'org.ametys.plugins.blog.Content.post'; } );
            
            if (contentTargets != null && contentTargets.length > 0)
            {
                for (var j = 0; j < contentTargets.length; j++)
                {
                    org.ametys.plugins.blog.ui.sitemap.Update.createdPostIds.push(contentTargets[j].getParameters().id);
                }
            }
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED)
        {
            var allTargets = messages[i].getTargets();
            var targets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'tag' } );
            var contentTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'content' && target.getParameters().type == 'org.ametys.plugins.blog.Content.post'; } );
            
            if (contentTargets != null && contentTargets.length > 0)
            {
                var contentIds = [];
                for (var j = 0; j < contentTargets.length; j++)
                {
                    var postId = contentTargets[j].getParameters().id;
                    if (org.ametys.plugins.blog.ui.sitemap.Update.createdPostIds.indexOf(postId) >= 0)
                    {
                        contentIds.push(postId);
                    }
                }
                
                var serverMessage = new org.ametys.servercomm.ServerMessage('blog', '/post-pages.xml', {contentIds: contentIds}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, org.ametys.plugins.blog.ui.sitemap.Update.postCreatedCb, this, []);
                org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
            }
            if (targets != null && targets.length > 0 && org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId != null)
            {
                var pageIds = [];
                for (var j = 0; j < targets.length; j++)
                {
                    var tagName = targets[j].getParameters().name;
                    
                    pageIds.push('blog-tag://' + tagName + '?rootId=' + org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId);
                }
                
                var pageTargets = org.ametys.messagebus.bus.MessageBuilder.getInstance().createTargets('page', {'ids': pageIds});
                
                var events = [org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage(org.ametys.ribbon.RibbonManager.EVENTTYPE_CREATED, null, pageTargets)];
                org.ametys.messagebus.MessageBus.getInstance().fireMessages(events);
            }
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_DELETING)
        {
        	if (org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId != null)
        	{
        		var allTargets = messages[i].getTargets();
                var contentTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'content' && target.getParameters().type == 'org.ametys.plugins.blog.Content.post'; } );
                var tagTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'tag'; } );
                
                if (contentTargets != null && contentTargets.length > 0)
                {
                    var contentIds = [];
                    for (var j = 0; j < contentTargets.length; j++)
                    {
                        contentIds.push(contentTargets[j].getParameters().id);
                    }
                    
                    var serverMessage = new org.ametys.servercomm.ServerMessage('blog', '/post-pages.xml', {contentIds: contentIds}, org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, org.ametys.plugins.blog.ui.sitemap.Update.getPostPagesTargets, this, []);
                    var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
                    
                    org.ametys.plugins.blog.ui.sitemap.Update.getPostPagesTargets(response);
                }
                if (tagTargets != null && tagTargets.length > 0)
                {
                    org.ametys.plugins.blog.ui.sitemap.Update.deletingTagPageTargets = {};
                    for (var j = 0; j < tagTargets.length; j++)
                    {
                        var tagId = tagTargets[j].getParameters().id;
                        var tagName = tagTargets[j].getParameters().name;
                        var pageId = 'blog-tag://' + tagName + '?rootId=' + org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId;
                        
                        var pageTarget = org.ametys.messagebus.bus.MessageBuilder.getInstance().createTarget('page', {'id': pageId});
                        
                        org.ametys.plugins.blog.ui.sitemap.Update.deletingTagPageTargets[tagId] = pageTarget;
                    }
                }
        	}
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_DELETED)
        {
        	if (org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId != null)
        	{
        		var allTargets = messages[i].getTargets();
                var contentTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'content' && target.getParameters().type == 'org.ametys.plugins.blog.Content.post'; } );
                var tagTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, function(target) { return target.getType() == 'tag'; } );
                
                var pageTargets = [];
                
                if (contentTargets != null && contentTargets.length > 0)
                {
                    var postPageTargets = org.ametys.plugins.blog.ui.sitemap.Update.deletingPostPageTargets;
                    org.ametys.plugins.blog.ui.sitemap.Update.deletingPostPageTargets = {};
                    
                    for (var j = 0; j < contentTargets.length; j++)
                    {
                        var contentId = contentTargets[j].getParameters().id;
                        var thePostPageTargets = postPageTargets[contentId];
                        if (thePostPageTargets != null)
                        {
                            for (var k = 0; k < thePostPageTargets.length; k++)
                            {
                            	if (thePostPageTargets[k] != null)
                                {
                            		pageTargets.push(thePostPageTargets[k]);
                                }
                            }
                        }
                    }
                }
                if (tagTargets != null && tagTargets.length > 0)
                {
                    var tagPageTargets = org.ametys.plugins.blog.ui.sitemap.Update.deletingTagPageTargets;
                    org.ametys.plugins.blog.ui.sitemap.Update.deletingTagPageTargets = {};
                    
                    for (var j = 0; j < tagTargets.length; j++)
                    {
                        var tagId = tagTargets[j].getParameters().id;
                        if (tagPageTargets[tagId] != null)
                        {
                        	pageTargets.push(tagPageTargets[tagId]);
                        }
                    }
                }
                
                if (pageTargets.length > 0)
                {
                    var event = org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage(org.ametys.ribbon.RibbonManager.EVENTTYPE_DELETED, null, pageTargets);
                    org.ametys.messagebus.MessageBus.getInstance().fireMessages([event], 1);
                }
        	}
            
        }
    }
}

org.ametys.plugins.blog.ui.sitemap.Update.postCreatedCb = function(response, args)
{
    var pageIds = [];
    var postNodes = response.selectNodes('posts/post');
    for (var i = 0; i < postNodes.length; i++)
    {
        var postId = postNodes[i].getAttribute('id');
        var pageNodes = postNodes[i].selectNodes('page');
        for (var j = 0; j < pageNodes.length; j++)
        {
            pageIds.push(pageNodes[j].getAttribute('id'));
        }
        
        org.ametys.plugins.blog.ui.sitemap.Update.createdPostIds.remove(postId);
    }
    
    if (pageIds.length > 0)
    {
        var pageTargets = org.ametys.messagebus.bus.MessageBuilder.getInstance().createTargets('page', {'ids': pageIds});
        var event = org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage(org.ametys.ribbon.RibbonManager.EVENTTYPE_CREATED, null, pageTargets);
        org.ametys.messagebus.MessageBus.getInstance().fireMessages([event], 1);
    }
}

org.ametys.plugins.blog.ui.sitemap.Update.getPostPagesTargets = function(response, args)
{
    org.ametys.plugins.blog.ui.sitemap.Update.deletingPostPageTargets = {};
    
    var postNodes = response.selectNodes('posts/post');
    for (var i = 0; i < postNodes.length; i++)
    {
        var pageIds = [];
        
        var postId = postNodes[i].getAttribute('id');
        var pageNodes = postNodes[i].selectNodes('page');
        for (var j = 0; j < pageNodes.length; j++)
        {
            pageIds.push(pageNodes[j].getAttribute('id'));
        }
        
        var pageTargets = org.ametys.messagebus.bus.MessageBuilder.getInstance().createTargets('page', {'ids': pageIds});
        
        org.ametys.plugins.blog.ui.sitemap.Update.deletingPostPageTargets[postId] = pageTargets;
    }
}

org.ametys.plugins.blog.ui.sitemap.Update.changeSitemapCb = function(response, args)
{
//    if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key='PLUGINS_BLOG_GET_ROOT_PAGE_ERROR'/>", response, 'org.ametys.plugins.blog.ui.sitemap.Update.changeSitemap'))
//    {
//        return;
//    }
    
    if (response != null)
    {
        var rootId = response.selectSingleNode('blog-root-page').getAttribute('id')
        if (rootId != null && rootId != '')
        {
            org.ametys.plugins.blog.ui.sitemap.Update.currentSitemapBlogRootPageId = rootId;
        }
    }
}

org.ametys.plugins.blog.ui.sitemap.Update.initialize();
